export class Main extends Indicator {

    constructor() {
        super();

        this.addInput("Step", InputType.float, 0.02);
        this.addInput("Maximum", InputType.float, 0.2);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.Buffer = this.addBuffer();

        this.save  = {
            lastreverse : 0,
            dirlong : 0,
            start : 0,
            last_high : 0,
            last_low : 0,
            ep : 0,
            sar : 0,
        };
    }

    onInit() {
        this.Buffer
            .setShape(Shape.Arrow)
            .setArrow(ArrowSymbols.Circle)
            .setColor(Color.Lime);
    }

    async onUpdate() {
        const {Step, Maximum, PriceType} = this.getInputs(),
            {
                High,
                Low
            } = await Bar.load([
                Bar.Mode.High,
                Bar.Mode.Low
            ], PriceType),
            barSize = Low.length;

        let first = true,
            dirlong, k,
            start, lastHigh, lastLow,
            ep, sar, priceLow, priceHigh, price,
            i, countedBars = Bar.counted();

        if (barSize < 3) return;

        if (countedBars === 0 || first) {
            i = barSize - 2;
            first = false;
            dirlong = true;
            start = Step;
            lastHigh = -10000000.0;
            lastLow = 10000000.0;
            this.save.lastreverse = 0;
            sar = 0;
            while (i > 0) {
                this.save.lastreverse = i;
                priceLow = Low[i];
                priceHigh = High[i];
                if (lastLow > priceLow)
                    lastLow = priceLow;
                if (lastHigh < priceHigh)
                    lastHigh = priceHigh;
                if (priceHigh > High[i+1] && priceLow > Low[i+1])
                    break;
                if (priceHigh < High[i+1] && priceLow < Low[i+1]) {
                    dirlong = false;
                    break;
                }
                i--;
            }
            k = i;
            while (k < barSize) {
                this.Buffer.set(k, 0);
                k++;
            }
            if (dirlong) {
                this.Buffer.set(i, Low[i+1]);
                ep = High[i];
            }
            else {
                this.Buffer.set(i, High[i+1]);
                ep = Low[i];
            }
            i--;
        }
        else {
            i = this.save.lastreverse;
            start = this.save.start;
            dirlong = this.save.dirlong;
            lastHigh = this.save.last_high;
            lastLow = this.save.last_low;
            ep = this.save.ep;
            sar = this.save.sar;
        }

        while (i >= 0) {
            priceLow = Low[i];
            priceHigh = High[i];
            price = this.Buffer.get(i + 1);
            if (dirlong && priceLow < price) {
                this.saveLastReverse(i + 1, true, start, priceLow, lastHigh, ep, sar);
                start = Step;
                dirlong = false;
                ep = priceLow;
                lastLow = priceLow;
                this.Buffer.set(i, lastHigh);
                i--;
                continue;
            }
            if (!dirlong && priceHigh > price) {
                this.saveLastReverse(i + 1, false, start, lastLow, priceHigh, ep, sar);
                start = Step;
                dirlong = true;
                ep = priceHigh;
                lastHigh = priceHigh;
                this.Buffer.set(i, lastLow);
                i--;
                continue;
            }
            sar = price + start * (ep - price);
            if (dirlong) {
                if (ep < priceHigh && (start + Step) <= Maximum)
                    start += Step;
                if (priceHigh < High[i+1] && i === barSize - 2)
                    sar = price;
                if (sar > Low[i+1]) sar = Low[i+1];
                let l = Low[i + 2] || 0;
                if (sar > l) sar = l;
                if (sar > priceLow) {
                    this.saveLastReverse(i + 1, true, start, priceLow, lastHigh, ep, sar);
                    start = Step;
                    dirlong = false;
                    ep = priceLow;
                    lastLow = priceLow;
                    this.Buffer.set(i, lastHigh);
                    i--;
                    continue;
                }
                if (ep < priceHigh) {
                    lastHigh = priceHigh;
                    ep = priceHigh;
                }
            }
            else {
                if (ep > priceLow && (start + Step) <= Maximum)
                    start += Step;
                if (priceLow < Low[i+1] && i === barSize - 2)
                    sar = price;
                if (sar < High[i+1]) sar = High[i+1];
                if (sar < High[i+1]) sar = High[i+1];
                if (sar < priceHigh) {
                    this.saveLastReverse(i + 1, false, start, lastLow, priceHigh, ep, sar);
                    start = Step;
                    dirlong = true;
                    ep = priceHigh;
                    lastHigh = priceHigh;
                    this.Buffer.set(i, lastLow);
                    i--;
                    continue;
                }
                if (ep > priceLow) {
                    lastLow = priceLow;
                    ep = priceLow;
                }
            }
            this.Buffer.set(i, sar);
            i--;
        }
    }

    saveLastReverse(last, dir, start, low, high, ep, sar) {
        this.save.lastreverse = last;
        this.save.dirlong = dir;
        this.save.start = start;
        this.save.last_low = low;
        this.save.last_high = high;
        this.save.ep = ep;
        this.save.sar = sar;
    }
}